# Bat-algorithm-optimized OCSVM for mineral prospectivity mapping
# Originated by Yongliang Chen

import numpy as np
import os,sys
from time import time
import pylab as p
import matplotlib.pyplot as plt
from numpy.random import *
import math
import struct
import copy
from sklearn import svm
from scipy.stats import pearsonr

#set the working directory
os.chdir(r'd:\\BatOCSVMPaper')
   
def choose_evidence(ncases,ndim,c,data,threshold,select = True):

    if select == True:
        cd = np.select([c <= threshold],[0], default = 1)
        n = cd.sum()
        da = np.zeros((ncases,n),float)
        c1 = np.zeros(n,float)
    
        j = 0
        for i in range(ndim):
           if cd[i] == 1:
               da[:,j] = data[:,i]
               c1[j] = c[i]
               j += 1
    else:
        da = data
        c1 = c  

    print "Contrasts for variables:"
    print c[1:]
    print "Selected variables:",n-1
    print "The contrasts for selected variables:"
    print c1[1:]
    
    # regularize weights in c1
    x = c1.sum()
    for i in range(n):
        c1[i] = c1[i]/x
        
    return n,c1,da

def compute_roc(bins,ncases,d,data,prob,textfilenamer,textfilenameg,textfilenamel):

    print "compute roc curve..."
    roc = np.zeros((bins+1,2),float)
    gain = np.zeros((bins+1,2),float)
    lift = np.zeros((bins+1,2),float)
    
    LR = np.zeros(bins+1,float)
    youden = np.zeros(bins+1,float)

    p = int(data[:,0].sum())
    n = d.sum() - p
    
    for j in range (2):
        roc[bins,j] = 1.0
        gain[bins,j] = 1.0
    lift[bins,0] = gain[bins,0]
    lift[bins,1] = 1.0/(1.0 + math.exp(-1.0))    
    #lift[0,1] = gain[0,0]
    
    minprob = prob.min()
    delta = (prob.max()-minprob)/float(bins)
    maxyouden = -99999.0
    maxLR = -99999.0
    
    for j in range(1,bins):
        tp = 0.0
        fp = 0.0
        tn = 0.0
        fn = 0.0

        threshold = minprob + delta*(bins - j)
        for i in range(ncases):
            if d[i] == 1:
                if prob[i] >= threshold and data[i,0] == 1.0:
                    tp += 1.0
                if  prob[i] >= threshold and data[i,0] == 0.0:
                    fp += 1.0
                if prob[i] < threshold and data[i,0] == 0.0:
                    tn += 1.0
                if  prob[i] < threshold and data[i,0] == 1.0:
                    fn += 1.0
                
        roc[j,0]= fp/(fp+tn)
        roc[j,1]= tp/(tp+fn)
        gain[j,0]= (tp+fp)/(p+n)#*100.0 #yrate
        gain[j,1]= roc[j,1]#*100.0
        lift[j,0]= gain[j,0]
        if gain[j,0] != 0.0:
            lift[j,1]= 1.0/(1.0 + math.exp(-gain[j,1]/gain[j,0]))
        
        youden[j] = roc[j,1] - roc[j,0]

        if youden[j] > maxyouden:
            maxyouden = youden[j]
            besthreshold = threshold
        
        if  roc[j,0] != 0:
            LR[j] =  1.0/(1.0 + math.exp(-roc[j,1]/roc[j,0]))
            if LR[j] > maxLR:
                maxLR = LR[j]
                besthresholdLR = threshold        
    asum = 0.0
    for i in range(ncases):
        if prob[i] >= besthreshold:
            asum = asum +1.0
    asum = asum/ncases        
            

    print textfilenamer,":"
    print "maximumYoudenIndex =",maxyouden,"besthreshold =",besthreshold, "Percent of target area =", asum
    #print "maximumLR =",maxLR,"besthresholdLR =",besthresholdLR
    
    textfile = open(textfilenamer,"w")
    for j in range(bins+1):
       textfile.write(str(minprob + delta*(bins - j)))
       textfile.write(",")
       for i in range(2):
          textfile.write(str(roc[j,i]))
          textfile.write(",")
       textfile.write(str(gain[j,0]))
       textfile.write(",")
       textfile.write(str(youden[j]))
       textfile.write(",")
       textfile.write(str(LR[j]))
       textfile.write("\n")
    textfile.close()
    
    textfile = open(textfilenameg,"w")
    for j in range(bins+1):
       #textfile.write(str(minprob + delta*(bins - j)))
       #textfile.write(",")
       textfile.write(str(gain[j,0]))
       textfile.write(",")
       textfile.write(str(gain[j,1]))          
       textfile.write("\n")
    textfile.close()

    textfile = open(textfilenamel,"w")
    for j in range(1,bins+1):
       #textfile.write(str(minprob + delta*(bins - j)))
       #textfile.write(",")
       textfile.write(str(lift[j,0]))
       textfile.write(",")
       textfile.write(str(lift[j,1])) 
       textfile.write("\n")
    textfile.close()
    
    #return besthreshold

def compute_roc_area(ncases,d,data,prob):
    print "compute the area under roc curve..."
    sample = d.sum()
    pd = data[:,0].sum()
    print "The number of known deposits:",pd
    
    na = 0
    area = 0.0
    for i in range(ncases):
        if d[i] == 1:
            if data[i,0] == 1.0:
                na += 1
                for j in range(ncases):
                    if d[j] == 1:
                        if data[j,0] == 0.0:
                            if prob[i] > prob[j]:
                                area += 1.0
                            elif prob[i] == prob[j]:
                                area += 0.5
                            else:
                                continue
                        else:
                            continue
            else:
                continue
    area = area/float(na*(sample-na))        
    q1 = area/(2.0-area)
    q2 = 2.0*area*area/(1.0+area)
    se = math.sqrt((area*(1-area)+(na - 1.0)*(q1 - area*area)+(sample-na-1.0)*(q2 - area*area))/(na*(sample-na)))
    z = (area - 0.5)/se
    aul = pd/(2*sample)+(1-pd/sample)*area
    
    return area,se,z,aul
    
def compute_rocweight(ncases,ndim,linNum,colNum,data,textfilename,Type,choose,row1,row2,col1,col2):

    print "compute rocweight..."
    rocweight = np.zeros((ndim-1,2),float)
    
    d = np.ones(ncases,int)#Ϊ0Ϊ1
    c = np.zeros(ndim,float)
    c[0] = 2.0

    sample = ncases
    for l in range(ncases):
        if data[l,:].sum() == 0.0:            
            sample = sample  - 1
            d[l] = 0
            
    #choose training cells from grid cells
    if choose == "yes":
        #Select training cells from every other line and collumns
        for i in range(linNum):
            if i % 2 == 0:
                for j in range(colNum):
                    if j % 2 == 0:
                        if data[i*colNum + j,0] == 0.0 and d[i*colNum + j] == 1:            
                            sample = sample  - 1
                            d[i*colNum + j] = 0    
        #Select training cells from certain block
        for i in range(row1):
            for j in range(colNum):
                if d[i*colNum + j] == 1:            
                    sample = sample  - 1
                    d[i*colNum + j] = 0 
        for i in range(row2,linNum):
            for j in range(colNum):
                if d[i*colNum + j] == 1:            
                    sample = sample  - 1
                    d[i*colNum + j] = 0
        for i in range(row1,row2):
            for j in range(col1):
                if d[i*colNum + j] == 1:            
                    sample = sample  - 1
                    d[i*colNum + j] = 0
            for j in range(col2,colNum):
                if d[i*colNum + j] == 1:            
                    sample = sample  - 1
                    d[i*colNum + j] = 0        

    if Type == "Youden":
        print "Yueden Indexes:"
    elif Type == "likelihoodratio":
        print "Likelihood Ratios:"
    else:
        print "Lift Indexes:"
        
    for i in range(1,ndim):
        tp = 0.0
        fp = 0.0
        tn = 0.0
        fn = 0.0        
        for j in range(ncases):
            if d[j] == 1:
                if data[j,i] == 1.0 and data[j,0] == 1.0:
                    tp += 1.0
                if data[j,i] == 1.0 and data[j,0] == 0.0:
                    fp += 1.0
                if data[j,i] == 0.0 and data[j,0] == 0.0:
                    tn += 1.0
                if data[j,i] == 0.0 and data[j,0] == 1.0:
                    fn += 1.0

        fpf0 = fp/(fp+tn)
        tpf0 = tp/(tp+fn)
        yrate0 = (tp+fp)/sample #(p+n)#lift0 = tp*(tp+fn+tn+fp)/((tp+fn)*(tp+fp))#math.log(tp*(tp+fn+tn+fp)/((tp+fn)*(tp+fp)))
        
        fpf1 = tn/(tn+fp)
        tpf1 = fn/(fn+tp)
        yrate1 = (fn +tn)/sample #fn*(tp+fn+tn+fp)/((fn+tp)*(fn+tn))#math.log(fn*(tp+fn+tn+fp)/((fn+tp)*(fn+tn)))
        
        if Type == "Youden":
            rocweight[i-1,0] = tpf0-fpf0
            rocweight[i-1,1] = tpf1-fpf1
            c[i] = tpf0-fpf0
            print rocweight[i-1,0],rocweight[i-1,1],c[i]
        elif Type == "likelihoodratio":
            if fpf0 != 0:
                rocweight[i-1,0] = 1.0/(1.0 + math.exp(-tpf0/fpf0))               
            if fpf1 != 0:
                rocweight[i-1,1] = 1.0/(1.0 + math.exp(-tpf1/fpf1))
            c[i] =  rocweight[i-1,0]           
            print rocweight[i-1,0],rocweight[i-1,1],c[i]
        else:
            if  yrate0 != 0:
                rocweight[i-1,0] = 1.0/(1.0 + math.exp(-tpf0/yrate0))                
            if  yrate1 != 0:     
                rocweight[i-1,1] = 1.0/(1.0 + math.exp(-tpf1/yrate1))
            c[i] = rocweight[i-1,0]    
            print rocweight[i-1,0],rocweight[i-1,1],c[i]            
        
    textfile = open(textfilename,"w")
    textfile.write("0.0")
    textfile.write(",")
    textfile.write("0.0")
    textfile.write("\n")
    for j in range(ndim-1):
       for i in range(2):
          textfile.write(str(rocweight[j,i]))
          if i < 1:
              textfile.write(",")
          else:
              textfile.write("\n")
    textfile.write("1.0")
    textfile.write(",")
    textfile.write("1.0")
    textfile.write("\n")
    textfile.close()

    return sample,d,rocweight,c

    
class FileInputOutput:

    def __init__(self,filename,k,l,bins,deposits,points,regionpoints,joints,xmin,xmax,ymin,ymax,ctype,permiterror = 0.0001):
        self.filename = filename
        self.k = k
        self.l = l
        self.bins = bins
        self.maxdeposits = deposits#100
        self.maxpoints = points #50000
        self.maxregionpoints = regionpoints#2000
        self.maxjoints = joints#3000
        self.type = ctype
        self.permiterror = permiterror#0.0001
    
    # Read grid file in binary format
    def input_binary_grid_file(self,filename):

        binaryfile = open(filename,"rb")

        HeadSign = binaryfile.read(4)
        print HeadSign
        
        if HeadSign == "DSBB":
            colNum, = struct.unpack("h",binaryfile.read(2))
            linNum, = struct.unpack("h",binaryfile.read(2))
            xmin, = struct.unpack("d",binaryfile.read(8))    
            xmax, = struct.unpack("d",binaryfile.read(8))
            ymin, = struct.unpack("d",binaryfile.read(8))
            ymax, = struct.unpack("d",binaryfile.read(8))
            zmin, = struct.unpack("d",binaryfile.read(8))
            zmax, = struct.unpack("d",binaryfile.read(8))
        
            ncases = colNum*linNum
            data = np.zeros(ncases,float)
            
            for i in range(ncases):
                x, = struct.unpack("f",binaryfile.read(4))
                data[i] = x
        elif HeadSign == "DSRB":
            HeadLength, = struct.unpack("l",binaryfile.read(4)) 
            Version, = struct.unpack("l",binaryfile.read(4))
            GridSign = binaryfile.read(4)
            print GridSign
            GridLength, = struct.unpack("l",binaryfile.read(4))
            linNum, = struct.unpack("l",binaryfile.read(4))
            colNum, = struct.unpack("l",binaryfile.read(4))
            xmin, = struct.unpack("d",binaryfile.read(8))    
            ymin, = struct.unpack("d",binaryfile.read(8))
            xsize, = struct.unpack("d",binaryfile.read(8))
            ysize, = struct.unpack("d",binaryfile.read(8))
            zmin, = struct.unpack("d",binaryfile.read(8))
            zmax, = struct.unpack("d",binaryfile.read(8))
            Rotation, = struct.unpack("d",binaryfile.read(8))
            BlankValue, = struct.unpack("d",binaryfile.read(8))
            DataSign = binaryfile.read(4)
            print DataSign
            DataLength, = struct.unpack("l",binaryfile.read(4))
            
            ncases = colNum*linNum
            data = np.zeros(ncases,float)
            
            for i in range(ncases):
                x, = struct.unpack("d",binaryfile.read(8))
                data[i] = x
            xmax = xmin + xsize*(colNum-1)
            ymax = ymin + ysize*(linNum-1)        
        binaryfile.close()

        print filename
        print "Data information:"
        print 'Samples:',ncases
        print
        print "Grid information:"
        print 'Columns:',colNum,',','Rows:',linNum
        print
        print "Coordinates:"
        print 'xmin=',xmin,',','xmax=',xmax
        print 'ymin=',ymin,',','ymax=',ymax
        print 'zmin=',zmin,',','zmax=',zmax
        print
        
        """
        for i in range(ncases):
            if (i+1)%10 == 0 or (i+1)%colNum == 0:
                print data[i],"\n"
            else:
                print data[i],
        """   
        return ncases,linNum,colNum,xmin,xmax,ymin,ymax,zmin,zmax,data

    def test_deposit_correlation(self,ncases,deposit,data):
    
        na = 0
        nb = 0
        for i in range(ncases):
            if data[i] < 1.70141E+038:
                if deposit[i] == 1.0:
                    na += 1
                else:
                    nb += 1
        print "na =",na,"nb=",nb
            
        area = 0.0
        for i in range(ncases):
            if data[i] < 1.70141E+038:
                if deposit[i] == 1.0:
                    for j in range(ncases):
                        if data[j] < 1.70141E+038:
                            if deposit[j] == 0.0:
                                if data[i] > data[j]:
                                    area += 1.0
                                elif data[i] == data[j]:
                                    area += 0.5
                                else:
                                    continue
        area = area/float(na*nb)        
        q1 = area/(2.0-area)
        q2 = 2.0*area*area/(1.0+area)
        se = math.sqrt((area*(1-area)+(na - 1.0)*(q1 - area*area)+(nb-1.0)*(q2 - area*area))/(na*nb))
        z = (area - 0.5)/se

        print "Area under roc curve =",area 
        print "Standard error of area =",se          
        print "Test statistics =",z 


    def extract_geochemical_anomaly(self,ncases,zmin,zmax,data,deposit):

        roc = np.zeros((self.bins+2,2),float)
        roc[0,0] = 0.0
        roc[0,1] = 0.0
        roc[self.bins+1,0] = 1.0
        roc[self.bins+1,1] = 1.0
     
        delta = (zmax - zmin)/float(self.bins)
        maxlift = -99999.0
        maxcontrast = -99999.0
        optimalThrsh = -99999.0
        
        for j in range(1,self.bins):
            tp = 0.0
            fp = 0.0
            tn = 0.0
            fn = 0.0

            threshold = zmin + delta*(bins - j)
            for i in range(ncases):
                if data[i] < 1.70141E+038:
                    if deposit[i] == 1.0 and data[i] >= threshold:
                        tp += 1.0
                    if  deposit[i] == 0.0 and data[i] >= threshold:
                        fp += 1.0
                    if deposit[i] == 0.0 and data[i]< threshold:
                        tn += 1.0
                    if  deposit[i] == 1.0 and data[i] < threshold:
                        fn += 1.0
            
            fpf = fp/(fp+tn)
            tpf = tp/(tp+fn)
            roc[j,0] = fpf
            roc[j,1] = tpf
            
            if self.type == "Youden":
                contrast = tpf - fpf
                if contrast > maxcontrast:
                    maxcontrast = contrast
                    optimalThresh = threshold
            elif self.type == "likelihoodratio":
                if fpf != 0.0:
                    contrast = tpf / fpf
                    if contrast > maxcontrast:
                        maxcontrast = contrast
                        optimalThresh = threshold
            else:
                pyrate = (tp+fp)/(tp+fn+tn+fp)
                lift = tp*(tp+fn+tn+fp)/((tp+fn)*(tp+fp))
                if lift > maxlift:
                    maxlift = lift
                    optimalThresh = threshold

        if self.type == "Youden":
            print "Maximum Youden index =",maxcontrast
        elif self.type == "likelihoodratio":
            print "Maximum likelihoodratio =",maxcontrast
        else:
            print "Maximum lift =",maxlift
        print "Optimal Threshold =",optimalThresh
        print "maximum value =", zmax

        binary = np.zeros(ncases,float)
            
        for i in range(ncases):
            if data[i] > optimalThresh and data[i] < 1.70141E+038:
                 binary[i] = 1.0
            else:
                 binary[i] = 0.0
       
        return binary,roc

    def input_deposit_data(self,filename):

         datafile = open(filename)
         deposit_co = np.zeros((self.maxdeposits,2),float)

         #Input the data from the *.mid file(including the deposit data)
         i = 0
         for line in datafile.readlines():
              """
              if line.find("CoordSys") == 0:
                   string1,string2,string3 = line.split("(")
                   string4,string5 = string2.split(")")
                   xmin,ymin = string4.split(", ")
                   xmin = float(xmin)
                   ymin = float (ymin)
                   string6,string7 = string3.split(")")
                   xmax,ymax = string6.split(", ")
                   xmax = float(xmax)
                   ymax = float(ymax)
              """
              if line.find("Point") == 0:
                   string,xcor,ycor = line.split(" ")
                   deposit_co[i, 0] = float(xcor)
                   deposit_co[i, 1] = float(ycor)
                   i += 1

         deposit_num = i
         
         print "xmin =",xmin,"ymin=",ymin
         print "xmax =",xmax,"ymax =",ymax
         print "deposit_num =",i
         
         for i in range(deposit_num):
              print deposit_co[i,0],deposit_co[i,1]
         
         datafile.close()
         
         return deposit_num,deposit_co

    def input_evidence_data(self,filename):
         
         datafile = open(filename)
         evidence_co = np.zeros((self.maxpoints,2),float)
         point_num = np.zeros(self.maxregionpoints,int)
         
         #Input the data from the *.mid file(including the evidence data)
         region_num = 0
         i = 0#line number of dataflie
         ii = 0#the number of line next to the line initiated with "Region"
         iii = 0#node number
         j = 0#region number
         num = 0#the number of region in one object
         logic = False
         for line in datafile.readlines():
             if line.find("Region") == 0:
                 string,num = line.split("  ")
                 num = int(num)
                 j = region_num +1
                 region_num += num
                 ii = i+1
                 logic = True
             if logic == True and i >= ii:
                 if j <= region_num:
                     if i == ii:
                         point_num[j-1] = int(line)
                     elif i > ii:
                         xcor,ycor = line.split(" ")
                         evidence_co[iii+i-ii-1, 0] = float(xcor)
                         evidence_co[iii+i-ii-1, 1] = float(ycor)
                         if i-ii == point_num[j-1]:
                             iii += point_num[j-1]
                             ii = i+1
                             j += 1
             i += 1
         """
         print "region_num =",region_num
         k = 0
         for i in range(region_num):
              print "Region 1"
              print point_num[i]
              for j in range(point_num[i]):
                  print evidence_co[k+j,0],evidence_co[k+j,1]
              k += point_num[i]
              print"\n"
         """     
         datafile.close()
         
         return region_num,point_num,evidence_co

    def generate_gridcells(self):
        """
        double ccc = (xmax - xmin) / (ymax - ymin)
        
        #The number of columns
        l = int(self.k * ccc)
        if l != 200:
            print "Error!, The number of collumn is wrong!"
            exit(0)
        """
        dx = (xmax - xmin) / float(self.l)
        dy = (ymax - ymin) / float(self.k)
        
        grid_co = np.zeros((self.k * self.l, 2),float)
        
        #Generate the uniform grid cells
        for i in range(self.k):
            for j in range(self.l):
                grid_co[i*self.l+j, 0] = xmin + j * dx + dx / 2.0
                grid_co[i*self.l+j, 1] = ymin + i * dy + dy / 2.0

        print "k =",self.k,"l=",self.l
        print "ncases=", self.k*self.l
        
        return grid_co,dx,dy,self.k*self.l


    def determine_orebearing_of_gridcells(self,deposit_co,deposit_num,xmin,ymin,dx,dy):

        griddata = np.zeros(self.k*self.l,float)
        #Determining grid cells whether containning a deposit
         
        for i in range (deposit_num):
            ii = int((deposit_co[i,0] - xmin)/dx)
            jj = int((deposit_co[i,1] - ymin)/dy)
            griddata[jj*self.l+ii] = 1.0

        return griddata
        
    def determine_evidence_existing_gridcells(self,region_num,point_num,evidence_co,grid_co,xmin,ymin,dx,dy):

        joint_co = np.zeros(self.maxjoints,float)
        grid_binary = np.zeros(self.k*self.l,float)
        xx1 = 0.0
        xx2 = 0.0
        xx3 = 0.0
        yy1 = 0.0
        yy2 = 0.0
        yy3 = 0.0
        ixx = 0
        iyy = 0
        
        #Determining map patterns whether exist in grid cells
        for ij in range(self.k):
            yg = grid_co[ij*self.l,1]#vertical coordinate of a grid cell
            kk = 0
            for i1 in range(region_num):
                kkks = 0
                for ii in range(kk, kk + point_num[i1]-1):
                    xx1 = evidence_co[ii, 0]
                    yy1 = evidence_co[ii, 1]
                    xx2 = evidence_co[ii + 1, 0]
                    yy2 = evidence_co[ii + 1, 1]
                    if ii < kk + point_num[i1] - 2:
                        xx3 = evidence_co[ii + 2, 0]
                        yy3 = evidence_co[ii + 2, 1]
                    elif ii == kk + point_num[i1] - 2:
                        xx3 = evidence_co[kk + 1, 0]
                        yy3 = evidence_co[kk + 1, 1]
                                          
                    #Determining the intersecting points of the scanning line and the polygon boundaries
                    if yy1 != yy2:
                        if (yg > yy1 and yg < yy2) or (yg < yy1 and yg > yy2):
                            joint_co[kkks] = (xx2 - xx1) * (yg - yy1) / (yy2 - yy1) + xx1
                            kkks += 1
                        elif yg == yy2:
                            if (yy3 > yg and yy1 < yg) or (yy3 < yg and yy1 > yg):
                                joint_co[kkks] = xx2
                                kkks += 1                                    
                            elif yy3 > yg and yy1 > yg:
                                joint_co[kkks] = xx2
                                joint_co[kkks+1] = xx2
                                kkks += 2                                    
                            elif yy3 < yg and yy1 < yg:
                                ixx = int((xx2 - xmin) / dx)
                                iyy = int((yy2 - ymin) / dy)
                                grid_binary[iyy * l + ixx] = 1.0
                                """
                                if grid_binary[iyy*l+ixx] == 0.0:
                                    grid_binary[iyy * l + ixx] = 1.0
                                elif grid_binary[iyy*l+ixx] == 1.0:
                                    grid_binary[iyy * l + ixx] = 0.0
                                """
                    elif yy1 == yg and yy2 == yg:
                
                        joint_co[kkks] = xx1
                        joint_co[kkks+1] = xx2
                        kkks += 2                 
                #Sort the X_coordinates
                for iii in range(kkks-1):
                    for jjj in range(iii + 1, kkks):
                        if joint_co[iii] > joint_co[jjj]:
                            xs = joint_co[iii]
                            joint_co[iii] = joint_co[jjj]
                            joint_co[jjj] = xs

                #Determining the grid_binary values
                kks = kkks / 2
                for iii in range(1,kks+1):
                    for jjj in range(self.l):
                        if grid_co[ij * self.l + jjj, 0] >= joint_co[2*iii-2] and grid_co[ij * self.l + jjj, 0]<= joint_co[2*iii-1]:
                            grid_binary[ij * self.l + jjj] = 1.0
                            """
                            if grid_binary[ij*l+jjj] == 0.0:
                                grid_binary[ij * l + jjj] = 1.0
                            elif grid_binary[ij*l+jjj] == 1.0:
                                grid_binary[ij * l + jjj] = 0.0
                            """
                kk += point_num[i1]

        return grid_binary

    def  construct_griddata_from_mif_and_grid_Files(self):

        namefile = open(self.filename)
        i = 0
        ip = 0
        ii = 0
        iii = 0
        iiii = 0
        iiiii = 0
        igc = 0
        ig = 0
       
        for line in namefile.readlines():
            i += 1
            if line.find("number of map patterns:")== 0:
                ii = i+1
            if i == ii:
                m = int(line)
                print "The number of map patterns:",m
            if line.find("mineral occurrence map:") == 0:
                iii = i+1
            if i == iii:
                #Open mineral deposit file and input the data
                string,s = line.split(".")
                string += ".mif"
                deposit_num,deposit_co = self.input_deposit_data(string)
                print string
                grid_co,dx,dy,ncases = self.generate_gridcells()
                grid_attribute = np.zeros((self.k*self.l,m+1),float)
                grid_attribute[:,0] = self.determine_orebearing_of_gridcells(deposit_co,deposit_num,xmin,ymin,dx,dy)
            if line.find("geochemical maps:") == 0:
                iiii = i+1
            if i == iiii:
                igc = int(line)
            if i>iiii and i<=iiii+igc:
                ip += 1
                string,s = line.split(".")
                string += ".grd"
                #Open binery grid file and input grid data
                ncases,linNum,colNum,xmin1,xmax1,ymin1,ymax1,zmin,zmax,data = self.input_binary_grid_file(string)
                #threshold = float(thresh)
                """
                print string
                if ncases != self.k*l or linNum != self.k or colNum != l or\
                   abs(xmin1 - xmin - dx/2.0 )> self.permiterror or abs(xmax1 - xmax + dx/2.0) > self.permiterror or \
                   abs(ymin1 - ymin - dy/2.0)> self.permiterror or abs(ymax1 - ymax + dy/2.0) > self.permiterror:
                    print "Error: the resolution of grid data doesn't match the that of MapInfo tables!"
                    quit()
                else:
                """
                #ncases = ncases-1
                self.test_deposit_correlation(ncases,grid_attribute[:,0],data)
                grid_attribute[:,ip],roc = self.extract_geochemical_anomaly(ncases,zmin,zmax,data,grid_attribute[:,0])
                s1,s2 = string.split(".")
                s1 += "roc.txt"
                print s1

                #Output the roc data into a text file
                self.output_roc_data(s1,roc)
                    
            if line.find("geological maps:") == 0:
                iiiii = i+1
            if i == iiiii:
                ig = int(line)
                #print "iiiii=",iiiii,"ig=",ig
            if i>iiiii and i<= iiiii+ig:
                ip += 1
                #Open evidence file and input the evidence data
                string,s = line.split(".")
                string += ".mif"
                region_num,point_num,evidence_co = self.input_evidence_data(string)
                print string
                grid_attribute[:,ip] = self.determine_evidence_existing_gridcells(region_num,\
                                        point_num,evidence_co,grid_co,xmin,ymin,dx,dy)
             
        namefile.close()
        
        #Preprocessing grid_attribute data
        ndim = m+1
        #grid_attribute = self.preprocessing_grid_attribute(ncases,ndim,grid_attribute)
        
        #Output the grid_attribute data into a text file
        self.output_grid_attribute_data("Attribute_data.txt",ncases,ndim,self.l,self.k,xmin,xmax,ymin,ymax,grid_attribute)
         
        return grid_attribute,ncases,ndim,self.k,self.l,dx,dy

    
    def input_gridcell_attribute_in_text_file(self,filename):
          
        textfile = open(filename) 
        i=0
        for line in textfile.readlines():
            if i == 0:
                ncases1,ndim1 = line.split(",")
                ncases = int(ncases1)
                ndim = int(ndim1)
                data = np.zeros((ncases,ndim),float)
            elif i == 1:
                colNum1,linNum1 = line.split(",")
                colNum = int(colNum1)
                linNum = int(linNum1)
            elif i == 2:
                xmin1,xmax1 = line.split(",")
                xmin = float(xmin1)
                xmax = float(xmax1)
            elif i == 3:
                ymin1,ymax1 = line.split(",")
                ymin = float(ymin1)
                ymax = float(ymax1)
            else:
                data[i-4,:] = line.split(",")
            i+=1
        textfile.close()

        print "Data information:"
        print 'Samples:',ncases,',','Variables:',ndim
        print
        print "Grid information:"
        print 'Columns:',colNum,',','Rows:',linNum
        print
        print "Coordinates:"
        print 'xmin=',xmin,',','xmax=',xmax
        print 'ymim=',ymin,',','ymax=',ymax
        print
        
        return ncases,ndim,linNum,colNum,xmin,xmax,ymin,ymax,data
    

    def output_grid_file(self,filename,colNum,linNum,xmin,xmax,ymin,ymax,result):

        gridfile = open(filename,"w")

        zmin = result.min()
        
        ncases = linNum*colNum
        zmax = -999999.0
        for i in range(ncases):
            if result[i]> zmax and result[i] < 1.70141E+038:
                zmax = result[i]

        gridfile.write("DSAA\n")
        gridfile.write(str(colNum))
        gridfile.write (" ")
        gridfile.write(str(linNum))
        gridfile.write("\n")
        gridfile.write(str(xmin))
        gridfile.write(" ")
        gridfile.write(str(xmax))
        gridfile.write("\n")
        gridfile.write(str(ymin))
        gridfile.write(" ")
        gridfile.write(str(ymax))
        gridfile.write("\n")
        gridfile.write(str(zmin))
        gridfile.write(" ")
        gridfile.write(str(zmax))
        gridfile.write("\n")
        
        for i in range(linNum):
            for j in range(colNum):
                if (j+1)%10 == 0 or j == colNum-1:
                    gridfile.write(str(result[i*colNum+j]))
                    gridfile.write("\n")
                else:
                    gridfile.write(str(result[i*colNum+j]))
                    gridfile.write(" ")
        gridfile.close()

    def output_roc_data(self,filename,roc):

        textfile = open(filename,"w")
        textfile.write(str(roc[0,0]))
        textfile.write(",")
        textfile.write(str(roc[0,1]))
        textfile.write(",")
        for ir in range(1,self.bins):
            textfile.write(str(roc[ir,0]))
            textfile.write(",")
            textfile.write(str(roc[ir,1]))
            textfile.write("\n")
        textfile.write(str(roc[self.bins+1,0]))
        textfile.write(",")
        textfile.write(str(roc[self.bins+1,1]))
        textfile.write("\n")
        textfile.close()
                            
    def output_grid_attribute_data(self,filename,ncases,ndim,colNum,linNum,xmin,xmax,ymin,ymax,grid_attribute):

        textfile = open(filename,"w")
        textfile.write(str(ncases))
        textfile.write(",")
        textfile.write(str(ndim))
        textfile.write("\n")
        textfile.write(str(colNum))
        textfile.write(",")
        textfile.write(str(linNum))
        textfile.write("\n")
        textfile.write(str(xmin))
        textfile.write(",")
        textfile.write(str(xmax))
        textfile.write("\n")
        textfile.write(str(ymin))
        textfile.write(",")
        textfile.write(str(ymax))
        textfile.write("\n")
        
        for j in range(ncases):
           for i in range(ndim):
              if i < ndim-1:
                  textfile.write(str(grid_attribute[j,i]))
                  textfile.write(",")
              else:           
                  textfile.write(str(grid_attribute[j,i]))              
                  textfile.write("\n")
        
        textfile.close()         
    
    def output_result_into_text_file(self,filename,probarea,likelifavarea,youdenfavarea,likeliarea,\
                                     youdenarea,orearea,obrarea,posarea,ndbpvarea,\
                                     se1,se2,se3,se4,se5,se6,se7,se8,se9,z1,z2,z3,z4,z5,z6,z7,z8,z9,\
                                     probthreshold,likelifavthreshold,youdenfavthreshold,likelindexthreshold,\
                                     youdenindexthreshold,orethreshold,obliqueratiothreshold,positfavthreshold,ndbpvthreshold):

        #output the results into a text file
        textfile = open(filename,"w")
        textfile.write("The areas under the roc curves:")
        textfile.write("\n")
        textfile.write("post probability area = ")
        textfile.write(str(probarea))
        textfile.write("\n")
        textfile.write("standard error =")
        textfile.write(str(se1))
        textfile.write("\n")
        textfile.write("statistical test index =")
        textfile.write(str(z1))
        textfile.write("\n")
        
        textfile.write("likelihood favorite area = ")
        textfile.write(str(likelifavarea)) 
        textfile.write("\n")
        textfile.write("standard error =")
        textfile.write(str(se2))
        textfile.write("\n")
        textfile.write("statistical test index =")
        textfile.write(str(z2))
        textfile.write("\n")
        
        textfile.write("youden favorite area = ")
        textfile.write(str(youdenfavarea)) 
        textfile.write("\n")
        textfile.write("standard error =")
        textfile.write(str(se3))
        textfile.write("\n")
        textfile.write("statistical test index =")
        textfile.write(str(z3))
        textfile.write("\n")

        textfile.write("likelihood index area = ")
        textfile.write(str(likeliarea)) 
        textfile.write("\n")
        textfile.write("standard error =")
        textfile.write(str(se4))
        textfile.write("\n")
        textfile.write("statistical test index =")
        textfile.write(str(z4))
        textfile.write("\n")
          
        textfile.write("youden index area = ")
        textfile.write(str(youdenarea)) 
        textfile.write("\n")
        textfile.write("standard error =")
        textfile.write(str(se5))
        textfile.write("\n")
        textfile.write("statistical test index =")
        textfile.write(str(z5))
        textfile.write("\n")

        textfile.write("orebearing index area = ")
        textfile.write(str(orearea)) 
        textfile.write("\n")
        textfile.write("standard error =")
        textfile.write(str(se6))
        textfile.write("\n")
        textfile.write("statistical test index =")
        textfile.write(str(z6))
        textfile.write("\n")

        textfile.write("obliqueratio area = ")
        textfile.write(str(obrarea)) 
        textfile.write("\n")
        textfile.write("standard error =")
        textfile.write(str(se7))
        textfile.write("\n")
        textfile.write("statistical test index =")
        textfile.write(str(z7))
        textfile.write("\n")

        textfile.write("positiverate area = ")
        textfile.write(str(posarea)) 
        textfile.write("\n")
        textfile.write("standard error =")
        textfile.write(str(se8))
        textfile.write("\n")
        textfile.write("statistical test index =")
        textfile.write(str(z8))
        textfile.write("\n")

        textfile.write("ndbpv area = ")
        textfile.write(str(posarea)) 
        textfile.write("\n")
        textfile.write("standard error =")
        textfile.write(str(se9))
        textfile.write("\n")
        textfile.write("statistical test index =")
        textfile.write(str(z9))
        textfile.write("\n")
        
        textfile.write("\n")
        textfile.write("The best thresholds:")
        textfile.write("\n")
        
        textfile.write("Threshold for post probability:")
        textfile.write(str(probthreshold))
        textfile.write("\n")

        textfile.write("Threshold for likelihood favorite:")
        textfile.write(str(likelifavthreshold))
        textfile.write("\n")

        textfile.write("Threshold for youden favorite:")
        textfile.write(str(youdenfavthreshold))
        textfile.write("\n")

        textfile.write("Threshold for likelihood index:")
        textfile.write(str(likelindexthreshold))
        textfile.write("\n")

        textfile.write("Threshold for youden index:")
        textfile.write(str(youdenindexthreshold))
        textfile.write("\n")
        
        textfile.write("Threshold for orebearing index:")
        textfile.write(str(orethreshold))
        textfile.write("\n")
        
        textfile.write("Threshold for obliqueratio:")
        textfile.write(str(obliqueratiothreshold))
        textfile.write("\n")

        textfile.write("Threshold for positiverate:")
        textfile.write(str(positfavthreshold))
        textfile.write("\n")

        textfile.write("Threshold for ndbpv:")
        textfile.write(str(ndbpvthreshold))
        textfile.write("\n")
        
        textfile.close()        


class ObjFunction:
    
    def __init__(self,vardim,chrom,bound,sample,da,deposit):#,incovar):

        self.vardim = vardim
        self.chrom = chrom
        self.bound = bound
        self.da = da
        self.deposit = deposit
        self.sample = sample
        #self.incovar = incovar

    def OCSVMFunc(self):
        
        
        clf = svm.OneClassSVM(kernel="rbf", gamma = self.chrom[0],nu = self.chrom[1],random_state = np.random.RandomState(1234))
        clf.fit(self.da)
        prob = - clf.decision_function(self.da)

                
        #print "compute the area under roc curve..."        
        na = 0
        area = 0.0
        for i in range(self.sample):
            if self.deposit[i] == 1.0:
                na += 1
                for j in range(self.sample):
                    if self.deposit[j] == 0.0:
                        if prob[i] > prob[j]:
                            area += 1.0
                        elif prob[i] == prob[j]:
                            area += 0.5
                        else:
                            continue
                    else:
                        continue
            else:
                continue
        fitness = area/float(na*(self.sample - na))

        print "gamm = ", self.chrom[0],"nu = ", self.chrom[1],"AUC = ",fitness

        return fitness
        
                

class BAIndividual:

    '''
    individual of bat algorithm
    '''

    def __init__(self,vardim,bound,sample,deposit,da):#,incovar):

        '''
        vardim: dimension of variables
        bound: boundaries of variables
        '''
        self.vardim = vardim
        self.bound = bound
        self.fitness = 0.
        self.trials = 0
        self.sample = sample
        self.deposit = deposit
        self.da = da
        #self.incovar = incovar
                

    def generate(self):
        '''
        generate a random chromsome for bat algorithm
        '''
        dim = self.vardim
        rnd = np.random.random(size=dim)
        self.chrom = np.zeros(dim)
        self.velocity = np.random.random(size=dim)
        for i in xrange(0, dim):
            self.chrom[i] = self.bound[0, i] + (self.bound[1, i] - self.bound[0, i]) * rnd[i]

    def calculateFitness(self):
        '''
        calculate the fitness of the chromsome
        '''

        obj = ObjFunction(self.vardim, self.chrom,self.bound,self.sample,self.da,self.deposit)#,self.incovar)
        self.fitness = obj.OCSVMFunc()
            
        
class BatAlgorithm:

    '''
    the class for bat algorithm
    '''

    def __init__(self, sizepop, vardim, bound, MAXGEN, params,sample,deposit,da):#,incovar):

        '''
        sizepop: population size
        vardim: dimension of variables
        bound: boundaries of variables
        MAXGEN: maximum iterations
        params: algorithm required parameters, it is a list which is consisting of[fmax, fmin, Amax, Amin, alpha, gamma]
        '''
        self.sizepop = sizepop
        self.vardim = vardim
        self.bound = bound
        self.MAXGEN = MAXGEN
        self.params = params
        self.population = []
        self.fitness = np.zeros(self.sizepop)
        self.freq = np.zeros(self.sizepop)
        self.loudness = np.zeros(self.sizepop)
        self.emissionrate = np.zeros(self.sizepop)
        self.initEmissionrate = np.zeros(self.sizepop)
        self.trace = np.zeros((self.MAXGEN, 2))
        self.sample = sample
        self.deposit = deposit
        self.da = da        
        #self.incovar = incovar

    def initialize(self):
        '''
        initialize the population of the bat algorithm
        
        '''
        
        for i in xrange(0, self.sizepop):
            ind = BAIndividual(self.vardim,self.bound,self.sample,self.deposit,self.da)#,self.incovar)
            ind.generate()
            self.population.append(ind)
            self.freq[i] = self.params[1] + (self.params[0] - self.params[1]) * np.random.random(1)
            self.loudness[i] = self.params[3] + (self.params[2] - self.params[3]) * np.random.random(1)
            self.initEmissionrate[i] = np.random.random(1)
            self.emissionrate[i] = self.initEmissionrate[i]

    def evaluation(self):
        '''
        evaluation the fitness of the population
        '''
        for i in xrange(0, self.sizepop):
            self.population[i].calculateFitness()
            self.fitness[i] = self.population[i].fitness

    def solve(self):

        '''
        the evolution process of the bat algorithm
        '''
        self.t = 0
        self.initialize()
        self.evaluation()
        bestIndex = np.argmax(self.fitness)
        self.best = copy.deepcopy(self.population[bestIndex])
        while self.t < self.MAXGEN:
            self.t += 1
            self.update()
            self.evaluation()
            best = np.max(self.fitness)
            bestIndex = np.argmax(self.fitness)
            if best > self.best.fitness:
                self.best = copy.deepcopy(self.population[bestIndex])

            self.avefitness = np.mean(self.fitness)
            self.trace[self.t - 1, 0] =  self.best.fitness
            self.trace[self.t - 1, 1] =  self.avefitness
            print("Generation %d: optimal function value is: %f; average function value is %f" % (
                self.t, self.trace[self.t - 1, 0], self.trace[self.t - 1, 1]))
        print("Optimal function value is: %f; " % self.trace[self.t - 1, 0])
        print "Optimal solution is:"
        print self.best.chrom
        self.printResult()
         

    def update(self):
        '''
        update the population
        '''
        
        for i in xrange(0, self.sizepop):
            self.freq[i] = self.params[1] + (self.params[0] - self.params[1]) * np.random.random(1)
            self.population[i].velocity += (self.best.chrom - self.population[i].chrom) * self.freq[i]
            self.population[i].chrom += self.population[i].velocity            
            for k in xrange(0, self.vardim):
                if self.population[i].chrom[k] < self.bound[0, k]:
                    self.population[i].chrom[k] = self.bound[0, k]
                if self.population[i].chrom[k] > self.bound[1, k]:
                    self.population[i].chrom[k] = self.bound[1, k]
            
            A = np.mean(self.loudness)
            if np.random.random(1) > self.emissionrate[i]:
                tmpInd = copy.deepcopy(self.best)                
                tmpInd.chrom += np.random.uniform(low = -1.0,high = 1.0, size = self.vardim) * A
                for k in xrange(0, self.vardim):
                    if tmpInd.chrom[k] < self.bound[0, k]:
                        tmpInd.chrom[k] = self.bound[0, k]
                    if tmpInd.chrom[k] > self.bound[1, k]:
                        tmpInd.chrom[k] = self.bound[1, k]
                tmpInd.calculateFitness()
                if tmpInd.fitness > self.best.fitness:
                    self.best = copy.deepcopy(tmpInd)
                    
            tmpi = copy.deepcopy(self.population[i])
            
            tmpi.chrom += np.random.uniform(low = -1.0,high = 1.0, size = self.vardim) * A
            for k in xrange(0, self.vardim):
                if tmpi.chrom[k] < self.bound[0, k]:
                    tmpi.chrom[k] = self.bound[0, k]
                if tmpi.chrom[k] > self.bound[1, k]:
                    tmpi.chrom[k] = self.bound[1, k]
                    
            tmpi.calculateFitness()
            if np.random.random(1) < self.loudness[i] and  tmpi.fitness > self.best.fitness:
                self.population[i] = tmpi
                self.best = copy.deepcopy(tmpi)
                self.loudness[i] *= self.params[4]
                self.emissionrate[i] = self.initEmissionrate[i] * (1 - np.exp(-1.0 * self.params[5] * self.t))


    def printResult(self):
        '''
        plot the result of bat algorithm
        '''
        x = np.arange(0, self.MAXGEN)
        y1 = self.trace[:, 0]
        y2 = self.trace[:, 1]
        plt.plot(x, y1, 'r', label='optimal value')
        plt.plot(x, y2, 'g', label='average value')
        plt.xlabel("Iteration")
        plt.ylabel("function value")
        plt.title("Bat algorithm for function optimization")
        plt.legend()
        plt.show()


if __name__ == '__main__':

    
    seed(12345)
    deposits = 100#the maximum number of the input mineral deposits
    points = 50000#the maximum number of nodes of one map pattern
    regionpoints = 2000#the maximum number of nodes of one region
    joints = 5000#the maximum number of intersected points of one scanning line with one region
    bins = 1000
    threshold = 0.01
    
    #map area
    xmin = 128.998744
    xmax = 129.501256
    ymin = 42.332047
    ymax = 42.666795
   
    k = 134 #k is the number of rows    bins = 1000
    l = 200 #int((xmax - xmin)/(ymax - ymin) * k + 0.5) #l is the number of collumns
    
    inputfile =  "FILE1.txt" #the text file lists the file names of all map layers
    output_text_file_name = "OutputData.txt"

    #construct class objects
    DataInAndOut = FileInputOutput(inputfile,k,l,bins,deposits,points,regionpoints,joints,xmin,xmax,ymin,ymax,"Youden")

    
    #Input data from mif files and grid files and construct attributes of the generated grid cells
    data,ncases,ndim,linNum,colNum,dx,dy = DataInAndOut.construct_griddata_from_mif_and_grid_Files()
    

    #compute rocweights and favorite indexes
    sample,d,Youdenweight,c = compute_rocweight(ncases,ndim,linNum,colNum,data,"Youdenweight1.txt",Type = "Youden",\
                                                choose = "no", row1 = 38, row2 = 163,col1 = 71,col2 = 192)
    #"yes" denotes choosing training cells
    #"no" denotes unchoosing training cells
    #"d" is used to indicate whether a cell is located in Blank area
    #row1,row2,col1,col2 are used to delineate training area
    
    print "sample =", sample

    
    #select significant evidences for mineral potential mapping
    ns,c1,da = choose_evidence(ncases,ndim,c,data,threshold)

    # Delete the collumn that contains mineral deposit data
    da1 = np.zeros((ncases, ns-1),float)
    for i in range(ncases):
        da1[i,:] = da[i,1:]

    c2 = np.zeros(ns-1,float)
    c2 = c1[1:] 

    # Delete the records corresponding the cells in blank areas
    ll = d.sum()
    da2 = np.zeros((ll,ns-1),float)
    dep = np.zeros(ll,float)
    l = 0
    for i in range(ncases):
        if d[i]== 1:
            da2[l,:] = da1[i,:]
            dep[l] = da[i,0]
            l = l + 1
    
    #####################################################

    print "OCSVM modeling......"
    start_time = time()

    # fit the OneSVM model_Default
    # nu---An upper bound on the fraction of training errors
    # and a lower bound of the fraction of support vectors.
    # Should be in the interval (0, 1]. By default 0.5 will be taken.
    # Kernel function type:'linear, poly, rbf, sigmoid,
    # gamma---Kernel coefficient for rbf, poly and sigmoid.
    # If gamma is auto then 1/n_features will be used instead.
    clf1 = svm.OneClassSVM(kernel="rbf",gamma = 1.0 /(ns - 1),nu = 0.5,random_state = np.random.RandomState(1234))
    clf1.fit(da2)

    OCSVM_distance = - clf1.decision_function(da2)
    distance = np.zeros(ncases,float)
    l = 0
    for i in range(ncases):
        if d[i]== 1:
            distance[i] = OCSVM_distance[l]
            l = l + 1

    DataInAndOut.output_grid_file('OCSVM_distance.grd',colNum,linNum,xmin+dx/2.0,xmax-dx/2.0,ymin+dy/2.0,ymax-dy/2.0,distance)
    compute_roc(bins,ncases,d,da,distance,"OCSVM_roc.txt","OCSVM_gain.txt","OCSVM_lift.txt")
    areaOCSVM,seOCSVM,zOCSVM,aulOCSVM = compute_roc_area(ncases,d,da,distance)
    print "areaOCSVM =", areaOCSVM,"seOCSVM =", seOCSVM,"zOCSVM =", zOCSVM,"aulOCSVM =",aulOCSVM 

    end_time = time()
    print "time =", end_time - start_time

    #########################################################################################################    
    print "Bat_OCSVM modeling......"

    start_time = time()
    
           
    bound = np.array([[0.00001,0.00001],[10.0,1.0]])
    
    #params: [fmax, fmin, Amax, Amin, alpha, gamma]
    ba = BatAlgorithm(20, 2, bound, 30, [1, 0, 1, 0, 0.9, 0.9],sample,dep,da2)
    ba.solve()
    print "gamma = ",ba.best.chrom[0], " mu = ",ba.best.chrom[1]

   
    clf = svm.OneClassSVM(kernel="rbf",gamma = ba.best.chrom[0],nu = ba.best.chrom[1],random_state = np.random.RandomState(1234))
    clf.fit(da2)
    
    #restore the original data
    Bat_OCSVM_distance = - clf.decision_function(da2)
    distance = np.zeros(ncases,float)
    l = 0
    for i in range(ncases):
        if d[i]== 1:
            distance[i] = Bat_OCSVM_distance[l]
            l = l + 1

    DataInAndOut.output_grid_file('Bat_OCSVM_distance.grd',colNum,linNum,xmin+dx/2.0,xmax-dx/2.0,ymin+dy/2.0,ymax-dy/2.0,distance)
    compute_roc(bins,ncases,d,da,distance,"Bat_OCSVM_roc.txt","Bat_OCSVM_gain.txt","Bat_OCSVM_lift.txt")
    areaBAOCSVM,seBAOCSVM,zBAOCSVM,aulBAOCSVM = compute_roc_area(ncases,d,da,distance)
    print "areaBAOCSVM =", areaBAOCSVM,"seBAOCSVM =", seBAOCSVM,"zBAOCSVM =", zBAOCSVM,"aulBAOCSVM =",aulBAOCSVM 

    end_time = time()
    print "time =", end_time - start_time
    
    #########################################################################################################    
    print "Bat_OCSVM modeling......"

    start_time = time()
    
           
    bound = np.array([[0.00001,0.00001],[10.0,1.0]])
    
    #params: [fmax, fmin, Amax, Amin, alpha, gamma]
    ba = BatAlgorithm(30, 2, bound, 20, [1, 0, 1, 0, 0.9, 0.9],sample,dep,da2)
    ba.solve()
    print "gamma = ",ba.best.chrom[0], " mu = ",ba.best.chrom[1]

   
    clf = svm.OneClassSVM(kernel="rbf",gamma = ba.best.chrom[0],nu = ba.best.chrom[1],random_state = np.random.RandomState(1234))
    clf.fit(da2)
    
    #restore the original data
    Bat_OCSVM_distance = - clf.decision_function(da2)
    distance = np.zeros(ncases,float)
    l = 0
    for i in range(ncases):
        if d[i]== 1:
            distance[i] = Bat_OCSVM_distance[l]
            l = l + 1

    DataInAndOut.output_grid_file('Bat_OCSVM_distance1.grd',colNum,linNum,xmin+dx/2.0,xmax-dx/2.0,ymin+dy/2.0,ymax-dy/2.0,distance)
    compute_roc(bins,ncases,d,da,distance,"Bat_OCSVM_roc1.txt","Bat_OCSVM_gain1.txt","Bat_OCSVM_lift1.txt")
    areaBAOCSVM,seBAOCSVM,zBAOCSVM,aulBAOCSVM = compute_roc_area(ncases,d,da,distance)
    print "areaBAOCSVM =", areaBAOCSVM,"seBAOCSVM =", seBAOCSVM,"zBAOCSVM =", zBAOCSVM,"aulBAOCSVM =",aulBAOCSVM 

    end_time = time()
    print "time =", end_time - start_time
    """
    #####################################################

    print "Compute Pearson's correlation coefficient......"
    start_time = time()

    # fit the OneSVM model_Default
    # nu---An upper bound on the fraction of training errors
    # and a lower bound of the fraction of support vectors.
    # Should be in the interval (0, 1]. By default 0.5 will be taken.
    # Kernel function type:'linear, poly, rbf, sigmoid,
    # gamma---Kernel coefficient for rbf, poly and sigmoid.
    # If gamma is auto then 1/n_features will be used instead.
    clf1 = svm.OneClassSVM(kernel="rbf",gamma = 1.755880,nu = 0.427568,random_state = np.random.RandomState(1234))
    clf1.fit(da2)

    OCSVM_X = - clf1.decision_function(da2)

    clf2 = svm.OneClassSVM(kernel="rbf",gamma = 1.360211,nu = 0.476444,random_state = np.random.RandomState(1234))
    clf2.fit(da2)

    OCSVM_Y = - clf2.decision_function(da2)

    print "Pearson's correlation coefficient =", pearsonr(OCSVM_X,OCSVM_Y)

    end_time = time()
    print "time =", end_time - start_time
    """

    print "Done."
